package com.qgs.dc.s7.service;

import com.qgs.dc.s7.enums.S7DriveManage;
import org.apache.plc4x.java.PlcDriverManager;
import org.apache.plc4x.java.api.PlcConnection;
import org.apache.plc4x.java.api.exceptions.PlcConnectionException;
import org.apache.plc4x.java.api.messages.PlcReadRequest;
import org.apache.plc4x.java.api.messages.PlcReadResponse;
import org.apache.plc4x.java.api.types.PlcResponseCode;
import org.apache.plc4x.java.api.value.PlcValue;
import org.apache.plc4x.java.utils.connectionpool.PooledPlcDriverManager;
import org.apache.plc4x.java.utils.connectionpool2.CachedDriverManager;
import org.eclipse.milo.opcua.sdk.client.OpcUaClient;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.stereotype.Component;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.concurrent.CompletableFuture;

/**
 * @Desc: ""
 * @Author: caixiang
 * @DATE: 2021/9/23 15:16
 */

@Component
public class S7Service {
    private final Logger logger = LoggerFactory.getLogger(getClass());

    private HashMap<String, PlcConnection> plcConnections = new HashMap<>();
    private PlcDriverManager driverManager;


    public S7Service(){
        driverManager = S7DriveManage.INSTANCE.getInstance();
    }

    public boolean addPlc(String url) throws PlcConnectionException {
        PlcConnection connection = driverManager.getConnection(url);
        return connection.isConnected();
    }

    //url : s7://192.168.0.200
    //address : %DB10:10.0:STRING(20)
    //get 一个或者多个value
    public Object getValue(String url,HashMap<String,String> listAddress) throws PlcConnectionException {
        try(PlcConnection conn = driverManager.getConnection(url)) {
            if(conn.isConnected()){
                if(conn.getMetadata().canRead()){
                    try {
                        PlcReadRequest.Builder builder = conn.readRequestBuilder();
                        for(String key:listAddress.keySet()){
                            builder.addItem(key, listAddress.get(key));
                        }

                        PlcReadRequest readRequest = builder.build();
                        CompletableFuture<? extends PlcReadResponse> execute = readRequest.execute();

                        PlcReadResponse response = execute.get();

                        for (String fieldName : response.getFieldNames()) {
                            if(response.getResponseCode(fieldName) == PlcResponseCode.OK) {
                                int numValues = response.getNumberOfValues(fieldName);
                                PlcValue asPlcValue = response.getAsPlcValue();
                                if(numValues == 1) {
                                    Object obj = response.getObject(fieldName);
                                    logger.info("Value[" + fieldName + "]: " + response.getObject(fieldName));
                                    return obj;
                                }
                                else {
                                    logger.info("Value[" + fieldName + "]:");
                                    List<Object> res = new ArrayList<>();
                                    for(int i = 0; i < numValues; i++) {
                                        logger.info(" - " + response.getObject(fieldName, i));
                                        res.add(response.getObject(fieldName,i));
                                    }
                                }
                            }
                            else {
                                logger.error("Error[" + fieldName + "]: " + response.getResponseCode(fieldName).name());
                            }
                        }

                    }catch (Exception e){
                        logger.error(e.getMessage());
                    }
                }else {
                    System.out.println("can not read");
                }
            }else {
                System.out.println("conn not connected");
            }

        }catch (Exception e){
            System.out.println(e.getMessage());
        }


        return driverManager.getConnection(url);
    }
}
